# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import six
import hl7
from hl7 import Segment, Field
from .compat import unittest
from .samples import sample_hl7


class ContainerTest(unittest.TestCase):
    def test_unicode(self):
        msg = hl7.parse(sample_hl7)
        self.assertEqual(six.text_type(msg), sample_hl7.strip())
        self.assertEqual(
            six.text_type(msg[3][3]),
            '1554-5^GLUCOSE^POST 12H CFST:MCNC:PT:SER/PLAS:QN'
        )

    def test_container_unicode(self):
        c = hl7.Container('|')
        c.extend(['1', 'b', 'data'])
        self.assertEqual(six.text_type(c), '1|b|data')


class MessageTest(unittest.TestCase):
    def test_segments(self):
        msg = hl7.parse(sample_hl7)
        s = msg.segments('OBX')
        self.assertEqual(len(s), 2)
        self.assertIsInstance(s[0], Segment)
        self.assertEqual(s[0][0:3], [['OBX'], ['1'], ['SN']])
        self.assertEqual(s[1][0:3], [['OBX'], ['2'], ['FN']])

        self.assertIsInstance(s[0][1], Field)

    def test_segments_does_not_exist(self):
        msg = hl7.parse(sample_hl7)
        self.assertRaises(KeyError, msg.segments, 'BAD')

    def test_segment(self):
        msg = hl7.parse(sample_hl7)
        s = msg.segment('OBX')
        self.assertEqual(s[0:3], [['OBX'], ['1'], ['SN']])

    def test_segment_does_not_exist(self):
        msg = hl7.parse(sample_hl7)
        self.assertRaises(KeyError, msg.segment, 'BAD')

    def test_segments_dict_key(self):
        msg = hl7.parse(sample_hl7)
        s = msg['OBX']
        self.assertEqual(len(s), 2)
        self.assertEqual(s[0][0:3], [['OBX'], ['1'], ['SN']])
        self.assertEqual(s[1][0:3], [['OBX'], ['2'], ['FN']])

    def test_get_slice(self):
        msg = hl7.parse(sample_hl7)
        s = msg.segments('OBX')[0]
        self.assertIsInstance(s, Segment)
        self.assertIsInstance(s[0:3], Segment)

    def test_ack(self):
        msg = hl7.parse(sample_hl7)
        ack = msg.create_ack()
        self.assertEqual(msg['MSH.1'], ack['MSH.1'])
        self.assertEqual(msg['MSH.2'], ack['MSH.2'])
        self.assertEqual('ACK', ack['MSH.9.1.1'])
        self.assertEqual(msg['MSH.9.1.2'], ack['MSH.9.1.2'])
        self.assertNotEqual(msg['MSH.7'], ack['MSH.7'])
        self.assertNotEqual(msg['MSH.10'], ack['MSH.10'])
        self.assertEqual('AA', ack['MSA.1'])
        self.assertEqual(msg['MSH.10'], ack['MSA.2'])
        self.assertEqual(20, len(ack['MSH.10']))
        self.assertEqual(msg['MSH.5'], ack['MSH.3'])
        self.assertEqual(msg['MSH.6'], ack['MSH.4'])
        self.assertEqual(msg['MSH.3'], ack['MSH.5'])
        self.assertEqual(msg['MSH.4'], ack['MSH.6'])
        ack2 = msg.create_ack(ack_code='AE', message_id='testid', application="python", facility="test")
        self.assertEqual('AE', ack2['MSA.1'])
        self.assertEqual('testid', ack2['MSH.10'])
        self.assertEqual('python', ack2['MSH.3'])
        self.assertEqual('test', ack2['MSH.4'])
        self.assertNotEqual(ack['MSH.10'], ack2['MSH.10'])


class TestMessage(hl7.Message):
    pass


class TestSegment(hl7.Segment):
    pass


class TestField(hl7.Field):
    pass


class TestRepetition(hl7.Repetition):
    pass


class TestComponent(hl7.Component):
    pass


class TestFactory(hl7.Factory):
    create_message = TestMessage
    create_segment = TestSegment
    create_field = TestField
    create_repetition = TestRepetition
    create_component = TestComponent


class FactoryTest(unittest.TestCase):
    def test_parse(self):
        msg = hl7.parse(sample_hl7, factory=TestFactory)
        self.assertIsInstance(msg, TestMessage)
        s = msg.segments('OBX')
        self.assertIsInstance(s[0], TestSegment)
        self.assertIsInstance(s[0](3), TestField)
        self.assertIsInstance(s[0](3)(1), TestRepetition)
        self.assertIsInstance(s[0](3)(1)(1), TestComponent)
        self.assertEqual("1554-5", s[0](3)(1)(1)(1))

    def test_ack(self):
        msg = hl7.parse(sample_hl7, factory=TestFactory)
        ack = msg.create_ack()
        self.assertIsInstance(ack, TestMessage)
        self.assertIsInstance(ack(1)(9), TestField)
        self.assertIsInstance(ack(1)(9)(1), TestRepetition)
        self.assertIsInstance(ack(1)(9)(1)(2), TestComponent)
        self.assertEqual("R01", ack(1)(9)(1)(2)(1))



import sys

# TODO Remove once 2.6 compat is removed
if sys.version_info < (3, 0):
    import unittest2 as unittest
else:
    import unittest

try:
    # Added in Python 3.3
    from unittest.mock import patch, Mock
except ImportError:
    from mock import patch, Mock


__all__ = ['unittest', 'patch', 'Mock']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from collections import namedtuple
import six
from .compat import python_2_unicode_compatible


@python_2_unicode_compatible
class Accessor(namedtuple('Accessor', ['segment', 'segment_num', 'field_num', 'repeat_num', 'component_num', 'subcomponent_num'])):
    __slots__ = ()

    def __new__(cls, segment, segment_num=1, field_num=None, repeat_num=None, component_num=None, subcomponent_num=None):
        """Create a new instance of Accessor for *segment*. Index numbers start from 1."""
        return super(Accessor, cls).__new__(cls, segment, segment_num, field_num, repeat_num, component_num, subcomponent_num)

    @property
    def key(self):
        """Return the string accessor key that represents this instance"""
        seg = self.segment if self.segment_num == 1 else self.segment + six.text_type(self.segment_num)
        return ".".join(six.text_type(f) for f in [seg, self.field_num, self.repeat_num, self.component_num, self.subcomponent_num] if f is not None)

    def __str__(self):
        return self.key

    @classmethod
    def parse_key(cls, key):
        """Create an Accessor by parsing an accessor key.

        The key is defined as:

            |   SEG[n]-Fn-Rn-Cn-Sn
            |       F   Field
            |       R   Repeat
            |       C   Component
            |       S   Sub-Component
            |
            |   *Indexing is from 1 for compatibility with HL7 spec numbering.*

        Example:

            |   PID.F1.R1.C2.S2 or PID.1.1.2.2
            |
            |   PID (default to first PID segment, counting from 1)
            |   F1  (first after segment id, HL7 Spec numbering)
            |   R1  (repeat counting from 1)
            |   C2  (component 2 counting from 1)
            |   S2  (component 2 counting from 1)
        """
        def parse_part(keyparts, index, prefix):
            if len(keyparts) > index:
                num = keyparts[index]
                if num[0].upper() == prefix:
                    num = num[1:]
                return int(num)
            else:
                return None

        parts = key.split('.')
        segment = parts[0][:3]
        if len(parts[0]) > 3:
            segment_num = int(parts[0][3:])
        else:
            segment_num = 1
        field_num = parse_part(parts, 1, 'F')
        repeat_num = parse_part(parts, 2, 'R')
        component_num = parse_part(parts, 3, 'C')
        subcomponent_num = parse_part(parts, 4, 'S')
        return cls(segment, segment_num, field_num, repeat_num, component_num, subcomponent_num)



"""Python 2/3 Compatibility Helper

Inspired by:

* https://docs.djangoproject.com/en/dev/topics/python3/
* http://lucumr.pocoo.org/2011/1/22/forwards-compatible-python/
* http://python-future.org/index.html
* http://docs.python.org/3.3/howto/pyporting.html

"""
import six


def python_2_unicode_compatible(cls):
    """
    Class decorator that provides appropriate Python 2 __unicode__ and __str__
    based upon Python 3' __str__.
    """
    if six.PY2:
        cls.__unicode__ = cls.__str__
        cls.__str__ = lambda self: self.__unicode__().encode('utf-8')
    return cls



from __future__ import unicode_literals
from hl7.datatypes import parse_datetime, _UTCOffset
from .compat import unittest
from datetime import datetime


class DatetimeTest(unittest.TestCase):
    def test_parse_date(self):
        self.assertEqual(datetime(1901, 2, 13), parse_datetime("19010213"))

    def test_parse_datetime(self):
        self.assertEqual(datetime(2014, 3, 11, 14, 25, 33), parse_datetime("20140311142533"))

    def test_parse_datetime_frac(self):
        self.assertEqual(datetime(2014, 3, 11, 14, 25, 33, 100000), parse_datetime("20140311142533.1"))
        self.assertEqual(datetime(2014, 3, 11, 14, 25, 33, 10000), parse_datetime("20140311142533.01"))
        self.assertEqual(datetime(2014, 3, 11, 14, 25, 33, 1000), parse_datetime("20140311142533.001"))
        self.assertEqual(datetime(2014, 3, 11, 14, 25, 33, 100), parse_datetime("20140311142533.0001"))

    def test_parse_tz(self):
        self.assertEqual(datetime(2014, 3, 11, 14, 12, tzinfo=_UTCOffset(330)), parse_datetime("201403111412+0530"))
        self.assertEqual(datetime(2014, 3, 11, 14, 12, 20, tzinfo=_UTCOffset(-300)), parse_datetime("20140311141220-0500"))

    def test_tz(self):
        self.assertEqual("+0205", _UTCOffset(125).tzname(datetime.utcnow()))
        self.assertEqual("-0410", _UTCOffset(-250).tzname(datetime.utcnow()))



# -*- coding: utf-8 -*-

"""
Primary version number source.

Forth element can be 'dev' < 'a' < 'b' < 'rc' < 'final'. An empty 4th
element is equivalent to 'final'.
"""
VERSION = (0, 3, 4, 'final')


def get_version():
    """Provide version number

    Use verlib format [1]_:
      N.N[.N]+[{a|b|c|rc}N[.N]+][.postN][.devN]

    .. [1] http://www.python.org/dev/peps/pep-0386/
    """
    main_version = '%s.%s.%s' % VERSION[0:3]

    if len(VERSION) < 4:
        return main_version

    version_type = VERSION[3]
    if not version_type or version_type == 'final':
        return main_version
    elif version_type == 'dev':
        return '%s.dev' % main_version
    else:
        return '%s%s' % (main_version, version_type)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import six

from .compat import unittest
import hl7
from .samples import rep_sample_hl7


SEP = '|^~\&'
CR_SEP = '\r'


class ConstructionTest(unittest.TestCase):
    def test_create_msg(self):
        # Create a message
        MSH = hl7.Segment(SEP[0], [hl7.Field(SEP[1], ['MSH'])])
        MSA = hl7.Segment(SEP[0], [hl7.Field(SEP[1], ['MSA'])])
        response = hl7.Message(CR_SEP, [MSH, MSA])
        response['MSH.F1.R1'] = SEP[0]
        response['MSH.F2.R1'] = SEP[1:]
        self.assertEqual(six.text_type(response), 'MSH|^~\\&|\rMSA')

    def test_append(self):
        # Append a segment to a message
        MSH = hl7.Segment(SEP[0], [hl7.Field(SEP[1], ['MSH'])])
        response = hl7.Message(CR_SEP, [MSH])
        response['MSH.F1.R1'] = SEP[0]
        response['MSH.F2.R1'] = SEP[1:]
        MSA = hl7.Segment(SEP[0], [hl7.Field(SEP[1], ['MSA'])])
        response.append(MSA)
        self.assertEqual(six.text_type(response), 'MSH|^~\\&|\rMSA')

    def test_append_from_source(self):
        # Copy a segment between messages
        MSH = hl7.Segment(SEP[0], [hl7.Field(SEP[1], ['MSH'])])
        MSA = hl7.Segment(SEP[0], [hl7.Field(SEP[1], ['MSA'])])
        response = hl7.Message(CR_SEP, [MSH, MSA])
        response['MSH.F1.R1'] = SEP[0]
        response['MSH.F2.R1'] = SEP[1:]
        self.assertEqual(six.text_type(response), 'MSH|^~\\&|\rMSA')
        src_msg = hl7.parse(rep_sample_hl7)
        PID = src_msg['PID'][0]
        self.assertEqual(six.text_type(PID), 'PID|Field1|Component1^Component2|Component1^Sub-Component1&Sub-Component2^Component3|Repeat1~Repeat2')
        response.append(PID)
        self.assertEqual(six.text_type(response), 'MSH|^~\\&|\rMSA\rPID|Field1|Component1^Component2|Component1^Sub-Component1&Sub-Component2^Component3|Repeat1~Repeat2')



# -*- coding: utf-8 -*-
"""python-hl7 is a simple library for parsing messages of Health Level 7
(HL7) version 2.x into Python objects.

* Documentation: http://python-hl7.readthedocs.org
* Source Code: http://github.com/johnpaulett/python-hl7
"""
from .version import get_version

__version__ = get_version()
__author__ = 'John Paulett'
__email__ = 'john -at- paulett.org'
__license__ = 'BSD'
__copyright__ = 'Copyright 2011, John Paulett <john -at- paulett.org>'

#: This is the HL7 Null value. It means that a field is present and blank.
NULL = '""'

from .parser import parse
from .containers import Sequence, Container, Message, Segment, Field, Repetition, Component, Factory
from .accessor import Accessor
from .util import ishl7, isfile, split_file, generate_message_control_id
from .datatypes import parse_datetime

__all__ = [
    "parse",
    "Sequence", "Container", "Message", "Segment", "Field", "Repetition", "Component", "Factory",
    "Accessor",
    "ishl7", "isfile", "split_file", "generate_message_control_id",
    "parse_datetime",
]



from hl7.client import MLLPClient, MLLPException, mllp_send, CR, SB, EB
from hl7 import __version__ as hl7_version
from optparse import Values
from shutil import rmtree
from tempfile import mkdtemp

from .compat import Mock, patch, unittest

import hl7
import os
import socket


class MLLPClientTest(unittest.TestCase):
    def setUp(self):
        # use a mock version of socket
        self.socket_patch = patch('hl7.client.socket.socket')
        self.mock_socket = self.socket_patch.start()

        self.client = MLLPClient('localhost', 6666)

    def tearDown(self):
        # unpatch socket
        self.socket_patch.stop()

    def test_connect(self):
        self.mock_socket.assert_called_once_with(socket.AF_INET,
                                                 socket.SOCK_STREAM)
        self.client.socket.connect.assert_called_once_with(('localhost', 6666))

    def test_close(self):
        self.client.close()
        self.client.socket.close.assert_called_once_with()

    def test_send(self):
        self.client.socket.recv.return_value = 'thanks'

        result = self.client.send('foobar\n')
        self.assertEqual(result, 'thanks')

        self.client.socket.send.assert_called_once_with('foobar\n')
        self.client.socket.recv.assert_called_once_with(4096)

    def test_send_message_unicode(self):
        self.client.socket.recv.return_value = 'thanks'

        result = self.client.send_message(u'foobar')
        self.assertEqual(result, 'thanks')

        self.client.socket.send.assert_called_once_with(b'\x0bfoobar\x1c\x0d')

    def test_send_message_bytestring(self):
        self.client.socket.recv.return_value = 'thanks'

        result = self.client.send_message(b'foobar')
        self.assertEqual(result, 'thanks')

        self.client.socket.send.assert_called_once_with(b'\x0bfoobar\x1c\x0d')

    def test_send_message_hl7_message(self):
        self.client.socket.recv.return_value = 'thanks'

        message = hl7.parse('MSH|^~\&|GHH LAB|ELAB')

        result = self.client.send_message(message)
        self.assertEqual(result, 'thanks')

        self.client.socket.send.assert_called_once_with(
            b'\x0bMSH|^~\&|GHH LAB|ELAB\x1c\x0d'
        )

    def test_context_manager(self):
        with MLLPClient('localhost', 6666) as client:
            client.send('hello world')

        self.client.socket.send.assert_called_once_with('hello world')
        self.client.socket.close.assert_called_once_with()

    def test_context_manager_exception(self):
        try:
            with MLLPClient('localhost', 6666):
                raise Exception()
            self.fail()
        except:
            # expected
            pass

        # socket.close should be called via the with statement
        self.client.socket.close.assert_called_once_with()


class MLLPSendTest(unittest.TestCase):
    def setUp(self):
        # patch to avoid touching sys and socket
        self.socket_patch = patch('hl7.client.socket.socket')
        self.mock_socket = self.socket_patch.start()
        self.mock_socket().recv.return_value = 'thanks'

        self.stdout_patch = patch('hl7.client.stdout')
        self.mock_stdout = self.stdout_patch.start()

        self.stdin_patch = patch('hl7.client.stdin')
        self.mock_stdin = self.stdin_patch.start()

        self.stderr_patch = patch('hl7.client.stderr')
        self.mock_stderr = self.stderr_patch.start()

        # we need a temporary directory
        self.dir = mkdtemp()
        self.write(SB + b'foobar' + EB + CR)

        self.option_values = Values({
            'port': 6661,
            'filename': os.path.join(self.dir, 'test.hl7'),
            'verbose': True,
            'loose': False,
            'version': False,
        })

        self.options_patch = patch('hl7.client.OptionParser')
        option_parser = self.options_patch.start()
        self.mock_options = Mock()
        option_parser.return_value = self.mock_options
        self.mock_options.parse_args.return_value = (self.option_values,
                                                     ['localhost'])

    def tearDown(self):
        # unpatch
        self.socket_patch.stop()
        self.options_patch.stop()
        self.stdout_patch.stop()
        self.stdin_patch.stop()
        self.stderr_patch.stop()

        # clean up the temp directory
        rmtree(self.dir)

    def write(self, content, path='test.hl7'):
        with open(os.path.join(self.dir, path), 'wb') as f:
            f.write(content)

    def test_send(self):
        mllp_send()

        self.mock_socket().connect.assert_called_once_with(('localhost', 6661))
        self.mock_socket().send.assert_called_once_with(
            SB + b'foobar' + EB + CR
        )
        self.mock_stdout.assert_called_once_with('thanks')

    def test_send_multiple(self):
        self.mock_socket().recv.return_value = 'thanks'
        self.write(SB + b'foobar' + EB + CR + SB + b'hello' + EB + CR)

        mllp_send()

        self.assertEqual(self.mock_socket().send.call_args_list[0][0][0],
                         SB + b'foobar' + EB + CR)
        self.assertEqual(self.mock_socket().send.call_args_list[1][0][0],
                         SB + b'hello' + EB + CR)

    def test_leftover_buffer(self):
        self.write(SB + b'foobar' + EB + CR + SB + b'stuff')

        self.assertRaises(MLLPException, mllp_send)

        self.mock_socket().send.assert_called_once_with(
            SB + b'foobar' + EB + CR
        )

    def test_quiet(self):
        self.option_values.verbose = False

        mllp_send()

        self.mock_socket().send.assert_called_once_with(
            SB + b'foobar' + EB + CR
        )
        self.assertFalse(self.mock_stdout.called)

    def test_port(self):
        self.option_values.port = 7890

        mllp_send()

        self.mock_socket().connect.assert_called_once_with(('localhost', 7890))

    def test_stdin(self):
        self.option_values.filename = None
        self.mock_stdin.return_value = FakeStream()

        mllp_send()

        self.mock_socket().send.assert_called_once_with(SB + b'hello' + EB + CR)

    def test_loose_no_stdin(self):
        self.option_values.loose = True
        self.option_values.filename = None
        self.mock_stdin.return_value = FakeStream()

        mllp_send()

        self.assertFalse(self.mock_socket().send.called)
        self.mock_stderr().write.assert_called_with(
            '--loose requires --file\n'
        )

    def test_loose_windows_newline(self):
        self.option_values.loose = True
        self.write(SB + b'MSH|^~\&|foo\r\nbar\r\n' + EB + CR)

        mllp_send()

        self.mock_socket().send.assert_called_once_with(
            SB + b'MSH|^~\&|foo\rbar' + EB + CR
        )

    def test_loose_unix_newline(self):
        self.option_values.loose = True
        self.write(SB + b'MSH|^~\&|foo\nbar\n' + EB + CR)

        mllp_send()

        self.mock_socket().send.assert_called_once_with(
            SB + b'MSH|^~\&|foo\rbar' + EB + CR
        )

    def test_loose_no_mllp_characters(self):
        self.option_values.loose = True
        self.write(b'MSH|^~\&|foo\r\nbar\r\n')

        mllp_send()

        self.mock_socket().send.assert_called_once_with(
            SB + b'MSH|^~\&|foo\rbar' + EB + CR
        )

    def test_loose_send_mutliple(self):
        self.option_values.loose = True
        self.mock_socket().recv.return_value = 'thanks'
        self.write(b'MSH|^~\&|1\r\nOBX|1\r\nMSH|^~\&|2\r\nOBX|2\r\n')

        mllp_send()

        self.assertEqual(self.mock_socket().send.call_args_list[0][0][0],
                         SB + b'MSH|^~\&|1\rOBX|1' + EB + CR)
        self.assertEqual(self.mock_socket().send.call_args_list[1][0][0],
                         SB + b'MSH|^~\&|2\rOBX|2' + EB + CR)

    def test_version(self):
        self.option_values.version = True

        mllp_send()

        self.assertFalse(self.mock_socket().connect.called)
        self.mock_stdout.assert_called_once_with(str(hl7_version))


class FakeStream(object):
    count = 0

    def read(self, buf):
        self.count += 1
        if self.count == 1:
            return SB + b'hello' + EB + CR
        else:
            return b''



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import six
import hl7
from hl7 import Accessor, Message, Segment, Field, Repetition, Component
from .compat import unittest
from .samples import sample_hl7, rep_sample_hl7, sample_file


class ParseTest(unittest.TestCase):
    def test_parse(self):
        msg = hl7.parse(sample_hl7)
        self.assertEqual(len(msg), 5)
        self.assertIsInstance(msg[0][0][0], six.text_type)
        self.assertEqual(msg[0][0][0], 'MSH')
        self.assertEqual(msg[3][0][0], 'OBX')
        self.assertEqual(
            msg[3][3],
            [[['1554-5'], ['GLUCOSE'], ['POST 12H CFST:MCNC:PT:SER/PLAS:QN']]]
        )
        # Make sure MSH-1 and MSH-2 are valid
        self.assertEqual(msg[0][1][0], '|')
        self.assertIsInstance(msg[0][1], hl7.Field)
        self.assertEqual(msg[0][2][0], '^~\&')
        self.assertIsInstance(msg[0][2], hl7.Field)
        # MSH-9 is the message type
        self.assertEqual(msg[0][9], [[['ORU'], ['R01']]])
        # Do it twice to make sure text coercion is idempotent
        self.assertEqual(six.text_type(msg), sample_hl7.strip())
        self.assertEqual(six.text_type(msg), sample_hl7.strip())

    def test_bytestring_converted_to_unicode(self):
        msg = hl7.parse(six.text_type(sample_hl7))
        self.assertEqual(len(msg), 5)
        self.assertIsInstance(msg[0][0][0], six.text_type)
        self.assertEqual(msg[0][0][0], 'MSH')

    def test_non_ascii_bytestring(self):
        # \x96 - valid cp1252, not valid utf8
        # it is the responsibility of the caller to convert to unicode
        msg = hl7.parse(b'MSH|^~\&|GHH LAB|ELAB\x963', encoding='cp1252')
        self.assertEqual(msg[0][4][0], 'ELAB\u20133')

    def test_non_ascii_bytestring_no_encoding(self):
        # \x96 - valid cp1252, not valid utf8
        # it is the responsibility of the caller to convert to unicode
        self.assertRaises(UnicodeDecodeError, hl7.parse,
                          b'MSH|^~\&|GHH LAB|ELAB\x963')

    def test_parsing_classes(self):
        msg = hl7.parse(sample_hl7)

        self.assertIsInstance(msg, hl7.Message)
        self.assertIsInstance(msg[3], hl7.Segment)
        self.assertIsInstance(msg[3][0], hl7.Field)
        self.assertIsInstance(msg[3][0][0], six.text_type)

    def test_nonstandard_separators(self):
        nonstd = 'MSH$%~\&$GHH LAB\rPID$$$555-44-4444$$EVERYWOMAN%EVE%E%%%L'
        msg = hl7.parse(nonstd)

        self.assertEqual(six.text_type(msg), nonstd)
        self.assertEqual(len(msg), 2)
        self.assertEqual(msg[1][5], [[['EVERYWOMAN'], ['EVE'], ['E'], [''], [''], ['L']]])

    def test_repetition(self):
        msg = hl7.parse(rep_sample_hl7)
        self.assertEqual(msg[1][4], [['Repeat1'], ['Repeat2']])
        self.assertIsInstance(msg[1][4], Field)
        self.assertIsInstance(msg[1][4][0], Repetition)
        self.assertIsInstance(msg[1][4][1], Repetition)
        self.assertEqual(six.text_type(msg[1][4][0][0]), 'Repeat1')
        self.assertIsInstance(msg[1][4][0][0], six.text_type)
        self.assertEqual(six.text_type(msg[1][4][1][0]), 'Repeat2')
        self.assertIsInstance(msg[1][4][1][0], six.text_type)

    def test_empty_initial_repetition(self):
        # Switch to look like "|~Repeat2|
        msg = hl7.parse(rep_sample_hl7.replace('Repeat1', ''))
        self.assertEqual(msg[1][4], [[''], ['Repeat2']])

    def test_subcomponent(self):
        msg = hl7.parse(rep_sample_hl7)
        self.assertEqual(
            msg[1][3],
            [[['Component1'], ['Sub-Component1', 'Sub-Component2'], ['Component3']]]
        )

    def test_elementnumbering(self):
        # Make sure that the numbering of repetitions. components and
        # sub-components is indexed from 1 when invoked as callable
        # (for compatibility with HL7 spec numbering)
        # and not 0-based (default for Python list)
        msg = hl7.parse(rep_sample_hl7)
        f = msg(2)(3)(1)(2)(2)
        self.assertIs(f, msg["PID.3.1.2.2"])
        self.assertIs(f, msg[1][3][0][1][1])
        f = msg(2)(4)(2)(1)
        self.assertIs(f, msg["PID.4.2.1"])
        self.assertIs(f, msg[1][4][1][0])
        # Repetition level accessed in list-form doesn't make much sense...
        self.assertIs(f, msg["PID.4.2"])

    def test_extract(self):
        msg = hl7.parse(rep_sample_hl7)

        # Full correct path
        self.assertEqual(msg['PID.3.1.2.2'], 'Sub-Component2')
        self.assertEqual(msg[Accessor('PID', 1, 3, 1, 2, 2)], 'Sub-Component2')

        # Shorter Paths
        self.assertEqual(msg['PID.1.1'], 'Field1')
        self.assertEqual(msg[Accessor('PID', 1, 1, 1)], 'Field1')
        self.assertEqual(msg['PID.1'], 'Field1')
        self.assertEqual(msg['PID1.1'], 'Field1')
        self.assertEqual(msg['PID.3.1.2'], 'Sub-Component1')

        # Longer Paths
        self.assertEqual(msg['PID.1.1.1.1'], 'Field1')

        # Incorrect path
        self.assertRaisesRegexp(IndexError, 'PID.1.1.1.2', msg.extract_field, *Accessor.parse_key('PID.1.1.1.2'))

        # Optional field, not included in message
        self.assertEqual(msg['MSH.20'], '')

        # Optional sub-component, not included in message
        self.assertEqual(msg['PID.3.1.2.3'], '')
        self.assertEqual(msg['PID.3.1.3'], 'Component3')
        self.assertEqual(msg['PID.3.1.4'], '')

    def test_assign(self):
        msg = hl7.parse(rep_sample_hl7)

        # Field
        msg['MSH.20'] = 'FIELD 20'
        self.assertEqual(msg['MSH.20'], 'FIELD 20')

        # Component
        msg['MSH.21.1.1'] = 'COMPONENT 21.1.1'
        self.assertEqual(msg['MSH.21.1.1'], 'COMPONENT 21.1.1')

        # Sub-Component
        msg['MSH.21.1.2.4'] = 'SUBCOMPONENT 21.1.2.4'
        self.assertEqual(msg['MSH.21.1.2.4'], 'SUBCOMPONENT 21.1.2.4')

        # Verify round-tripping (i.e. that separators are correct)
        msg2 = hl7.parse(six.text_type(msg))
        self.assertEqual(msg2['MSH.20'], 'FIELD 20')
        self.assertEqual(msg2['MSH.21.1.1'], 'COMPONENT 21.1.1')
        self.assertEqual(msg2['MSH.21.1.2.4'], 'SUBCOMPONENT 21.1.2.4')

    def test_unescape(self):
        msg = hl7.parse(rep_sample_hl7)

        # Replace Separators
        self.assertEqual(msg.unescape('\\E\\'), '\\')
        self.assertEqual(msg.unescape('\\F\\'), '|')
        self.assertEqual(msg.unescape('\\S\\'), '^')
        self.assertEqual(msg.unescape('\\T\\'), '&')
        self.assertEqual(msg.unescape('\\R\\'), '~')

        # Replace Highlighting
        self.assertEqual(msg.unescape('\\H\\text\\N\\'), '_text_')

        # Application Overrides
        self.assertEqual(msg.unescape('\\H\\text\\N\\', {'H': '*', 'N': '*'}), '*text*')

        # Hex Codes
        self.assertEqual(msg.unescape('\\X20202020\\'), '    ')

    def test_escape(self):
        msg = hl7.parse(rep_sample_hl7)

        self.assertEqual(msg.escape('\\'), '\\E\\')
        self.assertEqual(msg.escape('|'), '\\F\\')
        self.assertEqual(msg.escape('^'), '\\S\\')
        self.assertEqual(msg.escape('&'), '\\T\\')
        self.assertEqual(msg.escape('~'), '\\R\\')

        self.assertEqual(msg.escape('áéíóú'), '\\Xe1\\\\Xe9\\\\Xed\\\\Xf3\\\\Xfa\\')

    def test_file(self):
        # Extract message from file
        self.assertTrue(hl7.isfile(sample_file))
        messages = hl7.split_file(sample_file)
        self.assertEqual(len(messages), 1)

        # message can be parsed
        msg = hl7.parse(messages[0])

        # message has expected content
        self.assertEqual([s[0][0] for s in msg], ['MSH', 'EVN', 'PID', 'PD1', 'NK1', 'PV1'])


class ParsePlanTest(unittest.TestCase):
    def test_create_parse_plan(self):
        plan = hl7.parser.create_parse_plan(sample_hl7)

        self.assertEqual(plan.separators, ['\r', '|', '~', '^', '&'])
        self.assertEqual(plan.containers, [Message, Segment, Field, Repetition, Component])

    def test_parse_plan(self):
        plan = hl7.parser.create_parse_plan(sample_hl7)

        self.assertEqual(plan.separator, '\r')
        con = plan.container([1, 2])
        self.assertIsInstance(con, Message)
        self.assertEqual(con, [1, 2])
        self.assertEqual(con.separator, '\r')

    def test_parse_plan_next(self):
        plan = hl7.parser.create_parse_plan(sample_hl7)

        n1 = plan.next()
        self.assertEqual(n1.separators, ['|', '~', '^', '&'])
        self.assertEqual(n1.containers, [Segment, Field, Repetition, Component])

        n2 = n1.next()
        self.assertEqual(n2.separators, ['~', '^', '&'])
        self.assertEqual(n2.containers, [Field, Repetition, Component])

        n3 = n2.next()
        self.assertEqual(n3.separators, ['^', '&'])
        self.assertEqual(n3.containers, [Repetition, Component])

        n4 = n3.next()
        self.assertEqual(n4.separators, ['&'])
        self.assertEqual(n4.containers, [Component])

        n5 = n4.next()
        self.assertTrue(n5 is None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import sys

# Avoid directly importing the module. Prevents potential circular
# references when dependency needs to be installed via setup.py, so it
# is not yet available to setup.py
exec(open('hl7/version.py').read())

tests_require = []
if sys.version_info < (3, 0):
    tests_require.extend([
        'unittest2>=0.5.1'
    ])
if sys.version_info < (3, 3):
    tests_require.extend([
        'mock==1.0.1'
    ])

setup(
    name='hl7',
    version=get_version(),  # noqa
    description='Python library parsing HL7 v2.x messages',
    long_description="""
    python-hl7 is a simple library for parsing messages of Health Level 7
    (HL7) version 2.x into Python objects.

    * Documentation: http://python-hl7.readthedocs.org
    * Source Code: http://github.com/johnpaulett/python-hl7
    """,
    author='John Paulett',
    author_email='john -at- paulett.org',
    url='http://python-hl7.readthedocs.org',
    license='BSD',
    platforms=['POSIX', 'Windows'],
    keywords=[
        'HL7', 'Health Level 7', 'healthcare', 'health care', 'medical record'
    ],
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Healthcare Industry',
        'Topic :: Communications',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages=['hl7'],
    install_requires=['six'],
    test_suite='tests',
    tests_require=tests_require,
    entry_points={
        'console_scripts': [
            'mllp_send=hl7.client:mllp_send',
        ],
    },
    zip_safe=True,
)



from hl7.version import get_version

from .compat import patch, unittest


class GetVersionTest(unittest.TestCase):
    @patch('hl7.version.VERSION', new=(0, 4, 1))
    def test_no_modifier(self):
        self.assertEqual('0.4.1', get_version())

    @patch('hl7.version.VERSION', new=(0, 4, 1, ''))
    def test_empty_modifier(self):
        self.assertEqual('0.4.1', get_version())

    @patch('hl7.version.VERSION', new=(0, 4, 1, None))
    def test_none_modifier(self):
        self.assertEqual('0.4.1', get_version())

    @patch('hl7.version.VERSION', new=(0, 4, 1, 'final'))
    def test_final(self):
        self.assertEqual('0.4.1', get_version())

    @patch('hl7.version.VERSION', new=(0, 4, 1, 'rc'))
    def test_rc(self):
        self.assertEqual('0.4.1rc', get_version())

    @patch('hl7.version.VERSION', new=(0, 4, 1, 'rc4'))
    def test_rc_num(self):
        self.assertEqual('0.4.1rc4', get_version())

    @patch('hl7.version.VERSION', new=(0, 4, 1, 'b'))
    def test_beta(self):
        self.assertEqual('0.4.1b', get_version())

    @patch('hl7.version.VERSION', new=(0, 4, 1, 'a'))
    def test_alpha(self):
        self.assertEqual('0.4.1a', get_version())

    @patch('hl7.version.VERSION', new=(0, 4, 1, 'dev'))
    def test_dev(self):
        self.assertEqual('0.4.1.dev', get_version())
